#!/usr/bin/env python

"""
Point2Play Source Code
Copyright (c) 2004 by TransGaming Technologies, Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.

"""

import log, cdrom
import os, re, sys, config

# these are optical drive letters to use. Note that G is left out on purpose - it is already 
# mapped by default.
optical_drive_letters = ['F', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z' ]


# If Point2Play is being launched by the packaged launch script, then it needs to be able
# to be launched from anywhere.  In this case, P2PPATH will already be set, and we can use
# that as a path to locate our files.
p2ppath = os.getenv("P2PPATH")
if p2ppath:
    AGPTEST_LOCATION = p2ppath + "/agp_test/agp_test"
    GLXINFO_LOCATION = p2ppath + "/tests/glxinfo"
else:
    AGPTEST_LOCATION = "./agp_test/agp_test"
    GLXINFO_LOCATION = "./tests/glxinfo"

# these are the widgets that we try to autodetect
system_data_widgets_list = ["cpu", "cpu_ghz", "memory", "videocard_manufacturer", "videocard_type",
     "videocard_ram", "agp_aperture_size", "videocard_driver_version", "soundcard", "soundcard_driver",
     "kernel", "x_version", "distro"]

os.environ['PATH'] = "/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/usr/bin/X11"
# Note: we assume grep, which, tail and awk are installed somewhere in this PATH

def round_to_nearest_power_of_two(number):
    """ Takes a number and approximates (upward) to the nearest power of two.
    example, given 503, this function will return 512."""
    # Warning! Given 513, it will return 1024. Is that right? FIXME

    approximation = 2
    while number > approximation:
        approximation = approximation * 2
    return approximation

def utility_exists(name):
    pipeFile = os.popen("which " + name)
    while pipeFile.read() != "": pass
    status = pipeFile.close()
    if status:
        log.logger.info("The utility '" + name + "' does not appear to be in any of the usual places.")
        return False
    else: 
        return True
    

def detect_cpu():
    if os.path.exists("/proc/cpuinfo"):
        pipeFile = os.popen("grep 'model name' /proc/cpuinfo | tail -n 1 | awk -F': ' '{print $2}'")
        cpu = pipeFile.readline()
        return cpu[:-1]
    return ""

def detect_cpu_ghz():
    if os.path.exists("/proc/cpuinfo"):
        # fixme - does /proc/cpuinfo *always* store this as MHz? 
        pipeFile = os.popen("grep 'MHz' /proc/cpuinfo | tail -n 1 | awk -F': ' '{print $2}'")
        cpu_mhz = pipeFile.readline()
        cpu_mhz = float(cpu_mhz[:-1])
        # now convert to ghz and round
        cpu_ghz = cpu_mhz / 1000
        return str(round(cpu_ghz, 2))
    return ""

def detect_memory():
    if os.path.exists("/proc/meminfo"):
        pipeFile = os.popen("grep MemTotal: /proc/meminfo | awk '{print $2}'")
        mem = pipeFile.readline()
        mem = float(mem[:-1])
        mem = mem / 1024
        return str(int(mem))
    if utility_exists("free"):
        pipeFile = os.popen("free | grep Mem | awk '{print $2}'")
        mem = pipeFile.readline()
        mem = float(mem[:-1])
        mem = mem / 1024
        return str(int(mem))
    return ""

def detect_videocard_manufacturer():
    if os.path.exists(GLXINFO_LOCATION):
        pipeFile = os.popen(GLXINFO_LOCATION + " | grep 'OpenGL vendor string' | awk -F': ' '{print $2}'")
        videocard_manufacturer = pipeFile.readline()
        return videocard_manufacturer[:-1]
    log.logger.warning(_("The glxinfo program seems to be missing from your Cedega package."))
    return ""

def detect_videocard_type():
    if os.path.exists(GLXINFO_LOCATION):
        pipeFile = os.popen(GLXINFO_LOCATION + " | grep 'OpenGL renderer string' | awk -F': ' '{print $2}'")
        cardtype = pipeFile.readline()
        return cardtype[:-1]
    log.logger.warning(_("The glxinfo program seems to be missing from your Cedega package."))
    return ""

def detect_videocard_ram():
    """ Searches for the VideoRAM line in various log files.
        Converts an assumed kByte amount to megs. FIXME - can I assume these are all expressed in K? """

    videoram = 0
    if os.path.exists("/var/log/XFree86.0.log"):
        pipeFile = os.popen("grep -s -i -h VideoRAM: /var/log/XFree86.0.log | awk -F: '{ print $3 }' | awk -F' ' '{ print $1 }'")
        videoram_line = pipeFile.readline()
        match = re.search('([0-9]+)',videoram_line)
        if match:
             videoram = match.group(1)
    if os.path.exists("/var/log/Xorg.0.log"):
        pipeFile = os.popen("grep -s -i -h VideoRAM: /var/log/Xorg.0.log | awk -F: '{ print $3 }' | awk -F' ' '{ print $1 }'")
        videoram_line = pipeFile.readline()
        status = pipeFile.close()
        if not status:
            match = re.search('([0-9]+)',videoram_line)
            if match:
                videoram = match.group(1)
    videoram = int(videoram)
    if videoram > 0:
        videoram = videoram / 1024
        videoram = round_to_nearest_power_of_two(videoram)
        return str(videoram)
    return ""

def detect_agp_aperture_size():
    """ Returns the agp aperture size."""

    if os.path.exists(AGPTEST_LOCATION):
        pipeFile = os.popen(AGPTEST_LOCATION)
        response = pipeFile.readline()
        match = re.search('N/A', response)
        if match:
            log.logger.info("In AGP Aperture Size Test: " + response)
            return "N/A"
        else:
            match = re.search('AGP Memory available:\s+(\d+) MB', response)
            if match:
                return match.group(1)
            else:
                return ""  # shouldn't happen
                log.logger.info("------------ Unexpected AGP Aperture Size Test output -----------------")
    log.logger.warning(_("The AGP Aperture Size Test seems to be missing from your Cedega package."))
    return ""

def detect_videocard_driver_version():
    if os.path.exists(GLXINFO_LOCATION):
        pipeFile = os.popen(GLXINFO_LOCATION + " | grep 'OpenGL version string' | awk -F': ' '{print $2}'")
        version = pipeFile.readline()
        return version[:-1]
    log.logger.warning(_("The glxinfo program seems to be missing from your Cedega package."))
    return ""

# This function returns whether or not OpenGL direct rendering is available.  This value is
# requested by the OpenGL Direct Rendering test.
def detect_opengl_direct_rendering():
    if os.path.exists(GLXINFO_LOCATION):
        pipeFile = os.popen(GLXINFO_LOCATION + " | grep 'direct rendering' | awk -F': ' '{print $2}'")
        version = pipeFile.readline()
        return version[:-1]
    log.logger.warning(_("The glxinfo program seems to be missing from your Cedega package."))
    return ""

# Add an addition check for alsa setups here, mainly to detect the only the first soundcard
# if there are more than one configured
def detect_soundcard():
    if os.path.exists("/proc/asound/cards"):
        pipeFile = os.popen("cat /proc/asound/cards | head -n2 | tail -n1")
        soundcard = pipeFile.readline().strip()
        pipeFile.close()
        return soundcard[:-1]		    
    if utility_exists("lspci"):
        pipeFile = os.popen("lspci -v |grep 'Multimedia audio controller' | awk -F': ' '{print $2}'")
        soundcard = pipeFile.readline()
        pipeFile.close()
        return soundcard[:-1]
    return ""

# Better way to detect if ALSA or OSS is used. We cannot assume that all alsa modules
# are just compiled as modules, because with kernel 2.6.x it's possible to compile them
# into the kernel
def detect_soundcard_driver():
    if os.path.exists("/proc/asound/cards"):
       if utility_exists("alsactl"):
          pipeFile = os.popen("alsactl -v | awk -F' ' '{ print $3 }'")
	  version = pipeFile.readline()
	  pipeFile.close()
	  return "ALSA Version " + version[:-1]
       return "ALSA"
    return "OSS"

def detect_kernel():
    if utility_exists("uname"):
        pipeFile = os.popen("uname -r")
        kernel = pipeFile.readline()
        return kernel[:-1]
    return ""

def detect_x_version():
    if utility_exists("X"):
        os.environ["SHELL"] = "/bin/sh"  # make sure we're using sh for the redirection of stderr
        pipeFile = os.popen("X -version 2>&1 | grep -si 'XFree86 Version'")
        xver = pipeFile.readline()
        status = pipeFile.close()
        if status:  # didn't find XFree86 - try Xorg
            os.environ["SHELL"] = "/bin/sh" 
            pipeFile = os.popen("X -version 2>&1 | grep -si 'X Window System Version'")
            xver = pipeFile.readline()
            status = pipeFile.close()
            if status: # didn't find "X Window System Version"
                os.environ["SHELL"] = "/bin/sh"
                pipeFile = os.popen("X -version 2>&1 | grep -si 'Xorg Version'")
                xver = pipeFile.readline()
                status = pipeFile.close()
                if status: # didn't find "Xorg" at all
                    return ""
                return xver[:-1] # return string containing Xorg
            match = re.search('X Window System Version(.*)$', xver)
            return "Xorg Version" + match.group(1)
        return xver[:-1]   # the xfree version
    return ""

def get_firstline(filename):
    """ Returns the first line of a file. """

    pipeFile = os.popen("head -n 1 " + filename)
    contents = pipeFile.readline()
    status = pipeFile.close()
    if status:
        return ""
    return contents[:-1]

def detect_distro():

    # Note: order is important. Don't use a dictionary here. 
    # For example: if we detected the distro was Debian, we would never detect Xandros 
    # FIXME - perhaps we can figure out a way later to detect Ubuntu?

    distro_list = [ "Xandros", "Mandrake", "Red Hat", "Gentoo", "Slackware", "SuSE", "Linspire", "Lindows",
                    "Knoppix", "Arch", "Debian" ]
    file_list = [ "/etc/xandros-desktop-version", "/etc/mandrake-release", "/etc/redhat-release",
                  "/etc/gentoo-release", "/etc/slackware-version", "/etc/SuSE-release",
                  "/etc/linspire-version", "/etc/lindowsos-version", "/etc/knoppix-version",
                  "/etc/arch-release", "/etc/debian_version" ]

    if os.path.exists("/bin/lsb_release"):
    	pipeFile = os.popen("/bin/lsb_release -i | awk -F':' '{print $2}'")
	distributor_line = pipeFile.readline().strip()
	pipeFile.close()
	pipeFile = os.popen("/bin/lsb_release -r | awk -F':' '{print $2}'")
	release_line = pipeFile.readline().strip()
	pipeFile.close()
	pipeFile = os.popen("/bin/lsb_release -c | awk -F':' '{print $2}'")
	codename_line = pipeFile.readline().strip()
	pipeFile.close()
	return distributor_line + " " + release_line + " (" + codename_line + ")"
    else:
     	for distro, a_file in zip(distro_list, file_list):
            if os.path.exists(a_file):
               line = get_firstline(a_file)
               match = re.search('^Version: (.*)$', line)
               if match:
                  line = match.group(1)  # gets rid of "Version: "
               match = re.search(distro, line)
               if match:
                  return line
               else:
                  return distro + " " + line   # prefix the distro name when it isn't in the file


def detect_optical_drives(Point2Play, force=False):
    """ Stores optical drive info in the global configuration delta file. 
        Later this will all be incorporated into the GUI. For now it is just
        done behind the scenes. This function can be called from the setup wizard.

        If force=False, grep for Drive info in the global delta file,
        If force=True, always run dbus. 
    """

    fstabOpticalDriveList = cdrom.getFstabOpticalDrivesList()
    if fstabOpticalDriveList:
        log.logger.info("Optical drives found in /etc/fstab.")
        return  # this will all be handled properly by the engine

    dbusOpticalDrivesList = cdrom.getDbusOpticalDrivesList()
    if dbusOpticalDrivesList:
        profile = Point2Play.P2PFOLDER + Point2Play.GLOBAL_DELTA_FILENAME
        if not os.access(profile, os.F_OK): 
            log.logger.warning(_("No global delta file - something's wrong."))
            return

        if not force:
            # if we aren't supposed to force a re-detect, check to see if there's drive info
            # check if the string 'Drive ' occurs in the global delta file
            pipeFile = os.popen("grep -s -m 1 'Drive ' " + Point2Play.P2PFOLDER + Point2Play.GLOBAL_DELTA_FILENAME)
            exitstatus = pipeFile.close()
            if not exitstatus:
                return # found some drive info - don't force the re-detection

        gdconfig = config.config_file( Point2Play.get_profile_load_list(None, None) )
        drives_count = 0
        for drive in dbusOpticalDrivesList:
            drive_name = "Drive " + optical_drive_letters[drives_count]
            gdconfig.set_option( drive_name, "Type", "cdrom", None)
            gdconfig.set_option( drive_name, "Device", drive.getDev(), None)
            gdconfig.set_option( drive_name, "FailReadOnly", "Y", None)
            drives_count = drives_count + 1
        dconfig = config.config_file( Point2Play.get_profile_load_list(None, None, 2) )
        gdconfig.write(profile, dconfig)
    else:
        log.logger.warning(_("No optical drives have been found."))

# the setup wizard and Global Options gui use these keys

def make_detected_system_info_dictionary():
    """ Returns a dictionary of system information. Keys are:
        cpu, cpu_ghz, memory, videocard_manufacturer, videocard_type,
        videocard_ram, agp_aperture_size, videocard_driver_version, soundcard, 
        soundcard_driver, kernel, x_version."""

    system_info = {
        "cpu" : detect_cpu(),
        "cpu_ghz": detect_cpu_ghz(),
        "memory": detect_memory(),
        "videocard_manufacturer": detect_videocard_manufacturer(),
        "videocard_type": detect_videocard_type(),
        "videocard_ram": detect_videocard_ram(),
        "agp_aperture_size": detect_agp_aperture_size(),
        "videocard_driver_version": detect_videocard_driver_version(),
        "soundcard": detect_soundcard(),
        "soundcard_driver": detect_soundcard_driver(),
        "kernel": detect_kernel(),
        "x_version": detect_x_version(),
        "distro" : detect_distro()
    }
    return system_info
